import 'package:animate_do/animate_do.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';
import 'package:jhentai/src/config/ui_config.dart';
import 'package:jhentai/src/extension/widget_extension.dart';
import 'package:jhentai/src/setting/network_setting.dart';

import '../../../routes/routes.dart';
import '../../../utils/route_util.dart';
import '../../../utils/toast_util.dart';
import '../eh/tagsets/tag_sets_page.dart';

class SettingNetworkPage extends StatelessWidget {
  final TextEditingController proxyAddressController = TextEditingController(text: NetworkSetting.proxyAddress.value);
  final TextEditingController connectTimeoutController = TextEditingController(text: NetworkSetting.connectTimeout.value.toString());
  final TextEditingController receiveTimeoutController = TextEditingController(text: NetworkSetting.receiveTimeout.value.toString());

  SettingNetworkPage({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(centerTitle: true, title: Text('networkSetting'.tr)),
      body: Obx(
        () => ListView(
          padding: const EdgeInsets.only(top: 16),
          children: [
            _buildEnableDomainFronting(),
            if (NetworkSetting.enableDomainFronting.isTrue) _buildHostMapping().fadeIn(),
            if (GetPlatform.isDesktop) _buildProxyAddress(),
            _buildPageCacheMaxAge(),
            _buildConnectTimeout(),
            _buildReceiveTimeout(),
          ],
        ),
      ),
    );
  }

  Widget _buildEnableDomainFronting() {
    return ListTile(
      title: Text('enableDomainFronting'.tr),
      subtitle: Text('bypassSNIBlocking'.tr),
      trailing: Switch(value: NetworkSetting.enableDomainFronting.value, onChanged: NetworkSetting.saveEnableDomainFronting),
    );
  }

  Widget _buildHostMapping() {
    return ListTile(
      title: Text('hostMapping'.tr),
      subtitle: Text('hostMappingHint'.tr),
      trailing: const Icon(Icons.keyboard_arrow_right).marginOnly(right: 4),
      onTap: () => toRoute(Routes.hostMapping),
    );
  }

  Widget _buildProxyAddress() {
    return ListTile(
      title: Text('proxyAddress'.tr),
      subtitle: Text('proxyAddressHint'.tr),
      trailing: Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          SizedBox(
            width: 150,
            child: TextField(
              controller: proxyAddressController,
              decoration: const InputDecoration(isDense: true, labelStyle: TextStyle(fontSize: 12)),
              textAlign: TextAlign.center,
            ),
          ),
          IconButton(
            onPressed: () {
              if (proxyAddressController.value.text.isEmpty) {
                toast('invalid'.tr);
                return;
              }
              NetworkSetting.saveProxyAddress(proxyAddressController.value.text);
              toast('saveSuccess'.tr);
            },
            icon: Icon(Icons.check, color: UIConfig.resumeButtonColor),
          ),
        ],
      ),
    );
  }

  Widget _buildPageCacheMaxAge() {
    return ListTile(
      title: Text('pageCacheMaxAge'.tr),
      subtitle: Text('pageCacheMaxAgeHint'.tr),
      trailing: DropdownButton<Duration>(
        value: NetworkSetting.pageCacheMaxAge.value,
        elevation: 4,
        alignment: AlignmentDirectional.centerEnd,
        onChanged: (Duration? newValue) => NetworkSetting.savePageCacheMaxAge(newValue!),
        items: [
          DropdownMenuItem(child: Text('oneMinute'.tr), value: const Duration(minutes: 1)),
          DropdownMenuItem(child: Text('tenMinute'.tr), value: const Duration(minutes: 10)),
          DropdownMenuItem(child: Text('oneHour'.tr), value: const Duration(hours: 1)),
          DropdownMenuItem(child: Text('oneDay'.tr), value: const Duration(days: 1)),
          DropdownMenuItem(child: Text('threeDay'.tr), value: const Duration(days: 3)),
        ],
      ),
    );
  }

  Widget _buildConnectTimeout() {
    return ListTile(
      title: Text('connectTimeout'.tr),
      subtitle: Text('needRestart'.tr),
      trailing: Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          SizedBox(
            width: 50,
            child: TextField(
              controller: connectTimeoutController,
              decoration: const InputDecoration(isDense: true, labelStyle: TextStyle(fontSize: 12)),
              textAlign: TextAlign.center,
              inputFormatters: [
                FilteringTextInputFormatter.digitsOnly,
                NumberRangeTextInputFormatter(minValue: 0),
              ],
            ),
          ),
          const Text('ms', style: TextStyle(fontSize: 15)),
          IconButton(
            onPressed: () {
              int? value = int.tryParse(connectTimeoutController.value.text);
              if (value == null) {
                return;
              }
              NetworkSetting.saveConnectTimeout(value);
              toast('saveSuccess'.tr);
            },
            icon: Icon(Icons.check, color: UIConfig.resumeButtonColor),
          ),
        ],
      ),
    );
  }

  Widget _buildReceiveTimeout() {
    return ListTile(
      title: Text('receiveTimeout'.tr),
      subtitle: Text('needRestart'.tr),
      trailing: Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          SizedBox(
            width: 50,
            child: TextField(
              controller: receiveTimeoutController,
              decoration: const InputDecoration(isDense: true, labelStyle: TextStyle(fontSize: 12)),
              textAlign: TextAlign.center,
              inputFormatters: [
                FilteringTextInputFormatter.digitsOnly,
                NumberRangeTextInputFormatter(minValue: 0),
              ],
            ),
          ),
          const Text('ms', style: TextStyle(fontSize: 15)),
          IconButton(
            onPressed: () {
              int? value = int.tryParse(receiveTimeoutController.value.text);
              if (value == null) {
                return;
              }
              NetworkSetting.saveReceiveTimeout(value);
              toast('saveSuccess'.tr);
            },
            icon: Icon(Icons.check, color: UIConfig.resumeButtonColor),
          ),
        ],
      ),
    );
  }
}
